#include "StdAfx.h"
#include "CDriver.h"


CDriver::CDriver( LPCTSTR Drivername, DWORD dwFlags): 
	m_hDevice(INVALID_HANDLE_VALUE),
	m_DriverName(Drivername),
	m_Flags(dwFlags)
{
	m_LastError = 0;

    m_schSCManager = OpenSCManager 
						(NULL,               // machine (NULL == local)
                        NULL,                 // database (NULL == default)
                        SC_MANAGER_ALL_ACCESS // access required
                        );

	if (!m_schSCManager)
		m_LastError = ::GetLastError();

}


CDriver::~CDriver()
{
	m_LastError = 0;

	if (!(m_Flags & CDRIVER_KEEP_HANDLE_ALIVE))
		if (m_hDevice != INVALID_HANDLE_VALUE)
			CloseDevice();

	if (!(m_Flags & CDRIVER_KEEP_DRIVER_ALIVE))
		Stop();

	if (!(m_Flags & CDRIVER_KEEP_SCM_ENTRY_ALIVE))
		Remove();

	CloseServiceHandle( m_schSCManager );
}


BOOL CDriver::Install(IN LPCTSTR ServiceExe, BOOL bAddPath )
{
    SC_HANDLE  schService;
	char fullName[_MAX_PATH];
	char *pfname = fullName;

	m_LastError = 0;

	if ( bAddPath )
	{
		GetModuleFileName( NULL, fullName, sizeof(fullName) );
		DWORD l = strlen( fullName );

		for (pfname=&fullName[l]; l>0; --l, --pfname)
		{
			if ( *pfname == '\\' )
				break;
		}
		
		*++pfname = '\0';
		strcat( pfname, ServiceExe );
	}
	else
	{
		strcpy( fullName, ServiceExe );
	}


    schService = CreateService (m_schSCManager,        // SCManager database
                                m_DriverName,           // name of service
                                m_DriverName,           // name to display
                                SERVICE_ALL_ACCESS,    // desired access
                                SERVICE_KERNEL_DRIVER, // service type
                                SERVICE_DEMAND_START,  // start type
                                SERVICE_ERROR_NORMAL,  // error control type
                                fullName,              // service's binary
                                NULL,                  // no load ordering group
                                NULL,                  // no tag identifier
                                NULL,                  // no dependencies
                                NULL,                  // LocalSystem account
                                NULL                   // no password
                                );

    if (schService == NULL)
    {
        m_LastError = ::GetLastError();

        if (m_LastError == ERROR_SERVICE_EXISTS)
			m_LastError = 0;
		else
			return FALSE;
    }

    CloseServiceHandle (schService);

    return TRUE;
}




BOOL CDriver::Start()
{
    SC_HANDLE  schService;
    BOOL       ret;

	m_LastError = 0;


    schService = OpenService (m_schSCManager,
                              m_DriverName,
                              SERVICE_ALL_ACCESS
                              );

    if (schService == NULL)
    {
        m_LastError = ::GetLastError();
        return FALSE;
    }

    ret = StartService (schService,    // service identifier
                        0,             // number of arguments
                        NULL           // pointer to arguments
                        );
    if (!ret)
    {
        m_LastError = ::GetLastError();

        if (m_LastError == ERROR_SERVICE_ALREADY_RUNNING)
		{
			m_LastError = 0;
			ret = TRUE;
		}
    }

    CloseServiceHandle (schService);

    return ret;
}




BOOL CDriver::Stop()
{
    SC_HANDLE       schService;
    BOOL            ret;
    SERVICE_STATUS  serviceStatus;

 
	m_LastError = 0;

	schService = OpenService (m_schSCManager,
                              m_DriverName,
                              SERVICE_ALL_ACCESS
                              );

    if (schService == NULL)
    {
        m_LastError = ::GetLastError();
        return FALSE;
    }

    ret = ControlService (schService,
                          SERVICE_CONTROL_STOP,
                          &serviceStatus
                          );
    if (!ret)
        m_LastError = ::GetLastError();


    CloseServiceHandle (schService);

    return ret;
}




BOOL CDriver::Remove()
{
    SC_HANDLE  schService;
    BOOL       ret;

 
	m_LastError = 0;

	schService = OpenService (m_schSCManager,
                              m_DriverName,
                              SERVICE_ALL_ACCESS
                              );

    if (schService == NULL)
    {
        m_LastError = ::GetLastError();
        return FALSE;
    }

    ret = DeleteService (schService);

    if (!ret)
        m_LastError = ::GetLastError();


    CloseServiceHandle (schService);

    return ret;
}



BOOL CDriver::OpenDevice(
						 LPCSTR dosDeviceName
						 )
{
    char     completeDeviceName[64] = "";
    
	if ( dosDeviceName == NULL) 
		dosDeviceName = m_DriverName;

	m_LastError = 0;

    strcat (completeDeviceName,
            "\\\\.\\"
            );

    strcat (completeDeviceName,
            dosDeviceName
            );

    m_hDevice = CreateFile (completeDeviceName,
                          GENERIC_READ | GENERIC_WRITE,
                          0,
                          NULL,
                          OPEN_EXISTING,
                          FILE_ATTRIBUTE_NORMAL,
                          NULL
                          );

    if (m_hDevice == INVALID_HANDLE_VALUE)
	{
		m_LastError = ::GetLastError(); //0xFFFFFFFF;
		return FALSE;
	}


	return TRUE;
}


BOOL CDriver::CloseDevice(){
	
	m_LastError = 0;

	if (!CloseHandle(m_hDevice))
	{
		m_LastError = ::GetLastError();
		return FALSE;
	}
	else
		m_hDevice = INVALID_HANDLE_VALUE;

	return TRUE;
}

