// DbgTrapView.cpp : implementation of the CDbgTrapView class
//

#include "stdafx.h"
#include "DbgTrap.h"

#include "DbgTrapDoc.h"
#include "DbgTrapView.h"
#include "MainFrm.H"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define OUR_TIMER	0x1

CMainFrame* 
GetMainFrame();

/////////////////////////////////////////////////////////////////////////////
// CDbgTrapView

IMPLEMENT_DYNCREATE(CDbgTrapView, CListView)

BEGIN_MESSAGE_MAP(CDbgTrapView, CListView)
	//{{AFX_MSG_MAP(CDbgTrapView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



LONG
SetRegistryValue(
				 LPCTSTR pName,		// Substring relative to APP_REG_KEY to get data from
				 PVOID	 pBuffer,	// Buffer that the data is returned in
				 DWORD	 size,		// Size of the buffer...# of bytes written on return
				 DWORD	 type		// Type of the data to be stored
);

LONG
GetRegistryValue(
				 LPCTSTR pName,		// Substring relative to APP_REG_KEY to get data from
				 PVOID	 pBuffer,	// Buffer that the data is returned in
				 LPDWORD pSize		// Size of the buffer...# of bytes written on return
				 );



/////////////////////////////////////////////////////////////////////////////
// CDbgTrapView construction/destruction

CDbgTrapView::CDbgTrapView()
{
}

CDbgTrapView::~CDbgTrapView()
{
}

BOOL CDbgTrapView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CListView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDbgTrapView drawing

void CDbgTrapView::OnDraw(CDC* pDC)
{
	CDbgTrapDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
}

void CDbgTrapView::OnInitialUpdate()
{
	CListView::OnInitialUpdate();

		//
	// Reset wierd registry settings
	//
	CDbgTrapDoc* pDoc = GetDocument();
	DWORD* pOptions = &pDoc->m_optionFlags;

	DWORD size = sizeof(*pOptions);
	DWORD ret = GetRegistryValue( _T("Options"), (PBYTE)&*pOptions, &size );
	if (ret  != ERROR_SUCCESS)
		*pOptions = 0;

	DWORD t = *pOptions;
	*pOptions = ~*pOptions;

	GetMainFrame()->OnOntop();
	GetMainFrame()->OnCaptureuser();
	GetMainFrame()->OnCapturekernel();

	*pOptions = t;


}

/////////////////////////////////////////////////////////////////////////////
// CDbgTrapView diagnostics

#ifdef _DEBUG
void CDbgTrapView::AssertValid() const
{
	CListView::AssertValid();
}

void CDbgTrapView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CDbgTrapDoc* CDbgTrapView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDbgTrapDoc)));
	return (CDbgTrapDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDbgTrapView message handlers
CDbgTrapView* lv = NULL;

void 
CALLBACK EXPORT 
Update(
   HWND hWnd,      // handle of CWnd that called SetTimer
   UINT nMsg,      // WM_TIMER   
   UINT nIDEvent,   // timer identification
   DWORD dwTime    // system time
   )
{
	DWORD	stopIndex= *lv->pCurrentIndex;
	BOOL	bWrapped = ( stopIndex < lv->prevIndex );

	if (bWrapped)
	{
		lv->AddItemLoop( lv->pInfo->numEntries );
		lv->prevIndex = 0;
	}

	lv->AddItemLoop( stopIndex );

	CMainFrame* pMF = (CMainFrame*)AfxGetMainWnd();
	CString s;
	s.Format("DbgTrap - %u Events", lv->count );


	if  ( pMF != NULL )
		pMF->SetWindowText( s );

}

int CDbgTrapView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CListView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	//
	// Initialize the columns
	//
	GetListCtrl().InsertColumn( INDEX_COUNT, _T("#"), LVCFMT_LEFT, 30);
	GetListCtrl().InsertColumn( INDEX_TIME, _T("Time"), LVCFMT_LEFT, 50);
	GetListCtrl().InsertColumn( INDEX_PROCESSNAME, _T("Process"), LVCFMT_LEFT, 80);
	GetListCtrl().InsertColumn( INDEX_ACTION, _T("Action"), LVCFMT_LEFT, 80);
	GetListCtrl().InsertColumn( INDEX_CALLER, _T("Caller"), LVCFMT_LEFT, 80);
	GetListCtrl().InsertColumn( INDEX_VALUE, _T("Value"), LVCFMT_LEFT, 80);

	ModifyStyle( 0, LVS_REPORT | LVS_SINGLESEL );
	ListView_SetExtendedListViewStyleEx( m_hWnd, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT );
	
    // 
	// Start the driver
	//

	m_pdd = new CDriver( _T("DbgTrap"), CDRIVER_KEEP_SCM_ENTRY_ALIVE );

		
	if ( !m_pdd->Install(_T("DbgTrap.sys")) ||
		 !m_pdd->Start() ||
		 !m_pdd->OpenDevice()
		 )
	{
		CString s;
		s.Format( _T("Error starting device driver: %d"), 
				m_pdd->GetLastError() 
				);
		MessageBox( s );
		return -1;
	}



	//
	// Get the buffer
	//
	DWORD ret = 0;

	m_pdd->DeviceIoControl( IOCTL_HOOK, 
							NULL,
							0,
							&pInfo,
							sizeof(pInfo),
							&ret
							);

	//
	// Massage the shared buffer
	//
	VERIFY( pInfo->sig = 'JOSE' );

	peb = (PDBGTRAP_EVENT)((DWORD)(pInfo)+pInfo->size);
	pCurrentIndex = &pInfo->index;
	prevIndex = 0;
	count = 0;



	//
	// Setup our cheese buffer reading routine
	//
	lv = this;
	m_timer = SetTimer( OUR_TIMER, 1000, Update );

	return 0;
}

void CDbgTrapView::OnDestroy() 
{
	CListView::OnDestroy();

	KillTimer( m_timer );
	
	delete m_pdd;	
}


char* actions[] = 
		{
			"",
			"DbgPrint",
			"DbgPrompt",
			"Load Symbols",
			"Unload Symbols",
			"<unknown>"
		};

void RemoveChar( char* string, char c )
{
	DWORD l = strlen(string);
	char* p = string;
	char* s = &string[l];

	for ( ; p<s; ++p )
	{
		if ( *p == c )
			*p = ' ';
	}
}
				

DWORD CDbgTrapView::AddItemLoop( DWORD stopIndex )
{
	CListCtrl&		list= GetListCtrl();
	PDBGTRAP_EVENT pce=&peb[prevIndex];
	DWORD i = 0;

	for ( ; prevIndex < stopIndex; ++prevIndex, ++count, ++pce )
	{
		CString s;
		i = list.InsertItem( count, "", 0 );

		ASSERT(i==count);

		s.Format( "%u", count );
		list.SetItemText( i, INDEX_COUNT, s );

		FILETIME ft;
		ft.dwHighDateTime = pce->time.HighPart;
		ft.dwLowDateTime = pce->time.LowPart;
		CTime time( ft );
		s = time.Format( "%I:%M:%S" );
		list.SetItemText( i, INDEX_TIME, s );

		list.SetItemText( i, INDEX_PROCESSNAME, pce->procName );
		list.SetItemText( i, INDEX_ACTION, actions[pce->action] );

		s.Format( "%x", pce->callingAddr );
		list.SetItemText( i, INDEX_CALLER, s );

		RemoveChar( pce->string, '\n' );
		RemoveChar( pce->string, '\t' );
		list.SetItemText( i, INDEX_VALUE, pce->string );

		if ( GetDocument()->m_optionFlags & OPTION_AUTOSCROLL )
			list.EnsureVisible( lv->count, FALSE );

	}

	return i;
}

