///////////////////////////////////////////////////////////////////
// LogFile.cpp - implementation file
///////////////////////////////////////////////////////////////////

#include "logFile.h"
#include "windows.h"
//-----------------------------------------------------------------
CLogFile::CLogFile( void )
      {
      m_mode=0;  m_level=0;  m_options=0;
      m_status=NOT_READY;    m_emptyFileFlag=0;
      }
//-----------------------------------------------------------------
CLogFile::~CLogFile( void ){ Stop(); }
//-----------------------------------------------------------------
long CLogFile::Start( char *file, long mode, long options, 
                      long level )
    {
    if (m_status!=NOT_READY) return ALREADY_STARTED;
    char path[_MAX_PATH];
    m_mode=mode;    m_options=options;
    m_level=level;      m_count=0;

    // Verify mode
    if (m_mode==(LOG_TO_EXPLICIT_PATH & LOG_TO_TEMP_DIRECTORY) ||
      m_mode==(LOG_TO_EXPLICIT_PATH & LOG_TO_DATE_TIME_FILENAME) ||
      m_mode==(LOG_TO_TEMP_DIRECTORY & LOG_TO_DATE_TIME_FILENAME) ||
      m_mode==(LOG_TO_EXPLICIT_PATH & LOG_TO_TEMP_DIRECTORY & 
             LOG_TO_DATE_TIME_FILENAME))
          return TOO_MANY_MODES_SPECIFIED;

    m_logToFileFlag=(m_mode & LOG_TO_EXPLICIT_PATH) ||
            (m_mode & LOG_TO_TEMP_DIRECTORY) ||
            (m_mode & LOG_TO_DATE_TIME_FILENAME);

      _ftime( &m_startTime );  // store start time
      
      path[0]='\0';
      char buffer[_MAX_PATH];
      if (m_mode & LOG_TO_TEMP_DIRECTORY)      // Use Temp Dir Mode
            {
            char *var;
            var = getenv( "TMP" );   
            if (strcmp(var,"")==0)
                  var = getenv( "TEMP" );  
            if (strcmp(var,"")!=0)
                  strcpy(buffer,var);

            strcat(path,buffer);
            if (path[strlen(path)-1]!='\\') strcat(path,"\\");
            if (!(m_mode & LOG_TO_DATE_TIME_FILENAME))      
                strcat(path,file);
            }
      else if (m_mode & LOG_TO_DATE_TIME_FILENAME ) // Date/Time 
            {
            if (!(m_mode & LOG_TO_TEMP_DIRECTORY)) 
                       strcat(path,file);
            if (path[strlen(path)-1]!='\\') strcat(path,"\\");
            time_t ltime;
            struct tm *today; 
            time( &ltime );
            today = localtime( &ltime );
            strftime( buffer, 00,"%d-%b-%y-%H-%M-%S.log",today);
            strcat(path,buffer);
            }
      else if (m_mode & LOG_TO_EXPLICIT_PATH)  // Explicit Path 
            strcpy(path,file);

      // open file if logging to disk
      if (m_logToFileFlag)  
            {
            char flags[2];
            strcpy(flags,"w");
            if (m_mode & APPEND_FILE) strcpy(flags,"a");
            else strcpy(flags,"w");
            m_outfile=_fsopen(path, flags, _SH_DENYNO );
            if (m_outfile!=NULL)
                  {
                  strcpy(m_file,path); m_status=READY;
                     fseek( m_outfile, 0L, SEEK_END );
                  if (ftell(m_outfile)==0) m_emptyFileFlag=1;
                  return SUCCESS;
                  }
            else return UNABLE_TO_OPEN_FILE;
            }
      else if (m_mode & LOG_TO_DEBUG)  // log only to Debug Window
            {
            m_status=READY;
            return SUCCESS;
            }
      else  // a valid mode must be specified
            return NO_LOGGING_MODE_SPECIFIED;
      }
//-----------------------------------------------------------------
long CLogFile::Start( long mode, long options, long level )
      {  
   // this overloaded method allows the filename to not be specified
      return Start("",mode,options,level);
      }
//-----------------------------------------------------------------
long CLogFile::LogMessage( char *msg, long level ) 
    { // This method performs the actual logging 
      // operations, not the formatting of the 
      // message.  This method is more efficient and 
      // can be used if no formatting is needed

    if (m_status==READY &&
          m_level>=0 && level<=m_level) 
            {
            m_count++;
            m_message[0]='\0'; 
            if (m_options & SHOW_COUNT) // log the message count
                  sprintf(m_message,"%4ld - ",m_count);
             struct _timeb currTime;
             _ftime( &currTime );  
            if (m_options & SHOW_TIMESTAMP) // log the timestamp
                  {
                  char *timeStr;   
                  timeStr = ctime( & ( currTime.time ) ); 
                  sprintf(m_buffer1,"%.15s.%3hu - ", 
                                &timeStr[4], currTime.millitm);
                  strcat(m_message,m_buffer1);
                  }
            if (m_options & SHOW_ELAPSED_TIME) 
                  {               // log the elapsed time
                  double elapsedTime;
                  elapsedTime = difftime(currTime.time,
                                         m_startTime.time );
                  elapsedTime+=(currTime.millitm-
                        m_startTime.millitm)*.001;
                  sprintf(m_buffer1,"(%.3lf) - ",elapsedTime);
                  strcat(m_message,m_buffer1);
                  }
            if (m_options & SHOW_LOG_LEVEL)   // log the level
                  {
                  sprintf(m_buffer1,"<%ld> - ",level);
                  strcat(m_message,m_buffer1); 
                  }
            if (m_options & INDENT_BY_LEVEL) 
                  { // indent message by level
                  sprintf(m_buffer1,"%*s",level*3," ");
                  strcat(m_message,m_buffer1);
                  }
            strcat(m_message,msg);
            strcat(m_message,"\n");
            if (m_logToFileFlag)  // write message to log file
                  {
                  fputs(m_message,m_outfile);            
                  fflush(m_outfile);
                  }

            if (m_mode & LOG_TO_DEBUG) 
                  { // send message to Debug Window
                  OutputDebugString(m_message);
                  }
            }
      return SUCCESS;
      }
//-----------------------------------------------------------------
long CLogFile::LogMessage( long level, char *formatStr, ... )
      {
      va_list argp;
      va_start(argp, formatStr);

      vsprintf(m_buffer,formatStr,argp);
      return LogMessage(m_buffer,level);
      }
//-----------------------------------------------------------------
long CLogFile::Stop( void )
      {
      if (m_logToFileFlag && m_status!=NOT_READY)  
            { // if log file is open
            fclose(m_outfile);
            m_outfile=NULL;
     // if nothing was written to the file, delete it
            if (m_emptyFileFlag && m_count==0 && 
               (m_mode & DELETE_IF_EMPTY))  
                  unlink(m_file);
            m_status=NOT_READY;
            return SUCCESS;
            }
      else return NOT_LOGGING;
      }
//-----------------------------------------------------------------
long CLogFile::Pause( void )
      {
      if (m_status!=READY) return NOT_LOGGING;
      m_status=PAUSED;
      return SUCCESS;
      }
//-----------------------------------------------------------------
long CLogFile::Continue( void )
      {
      if (m_status!=PAUSED) return NOT_PAUSED;
      m_status=READY;
      return SUCCESS;
      }
//-----------------------------------------------------------------
void CLogFile::SetLevel( long level ){ m_level=level; }
//-----------------------------------------------------------------
long CLogFile::GetMode( void ){ return m_mode; }
//-----------------------------------------------------------------
long CLogFile::GetLevel( void ){ return m_level; }
//-----------------------------------------------------------------
long CLogFile::GetStatus( void ){ return m_status; }
//-----------------------------------------------------------------
char *CLogFile::GetFilename( void ){ return m_file; }

