///////////////////////////////////////////////////////////////////
// LogFile.h - header file
///////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <time.h>
#include <share.h>
#include <ctype.h>
#include <io.h>
#include <sys/timeb.h>

//=================================================================
// CLogFile class provides generic logging capabilities

class CLogFile
      {
      protected:
          FILE *m_outfile;
          long m_options, m_level, m_status, m_mode, m_count;
          struct _timeb m_startTime;   
          char m_file[_MAX_PATH];
          long m_emptyFileFlag, m_logToFileFlag;
          char m_message[1024], m_buffer[1024], m_buffer1[1024];

      public:
            enum mode_values
                  { LOG_TO_EXPLICIT_PATH=1,
                    LOG_TO_TEMP_DIRECTORY=2, 
                    LOG_TO_DATE_TIME_FILENAME=4, 
                    DELETE_IF_EMPTY=8,
                    APPEND_FILE=16, LOG_TO_DEBUG=64 };
            enum option_values
                   { SHOW_NONE=0, SHOW_TIMESTAMP=1, 
                     SHOW_ELAPSED_TIME=2, SHOW_LOG_LEVEL=4, 
                     SHOW_COUNT=8, SHOW_ALL=15, 
                     INDENT_BY_LEVEL=256 };
            enum status_values       
                   { NOT_READY, READY, PAUSED };
            enum return_values
                   { SUCCESS, UNABLE_TO_OPEN_FILE,
                     NOT_LOGGING, NOT_PAUSED,
                     ALREADY_STARTED, 
                     NO_LOGGING_MODE_SPECIFIED,
                     TOO_MANY_MODES_SPECIFIED };

            CLogFile( void );
            ~CLogFile( void );
            long Start( char *file, 
                  long type=LOG_TO_EXPLICIT_PATH | LOG_TO_DEBUG,
                  long options=SHOW_ALL, long level=0 );
            long Start( long type=LOG_TO_DEBUG, long 
                        options=SHOW_ALL, long level=0 );
            long Pause();
            long Continue();
            long Stop( void );
            long LogMessage( char *msg, long level=0 );
            long LogMessage( long level, char *formatStr, ... );
            void SetLevel(long level);
            long GetLevel( void );
            long GetMode( void );
            void SetOptions( long mode );
            void GetOptions( long mode );
            long GetStatus( void );
            char *GetFilename( void );
      };

