#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include <string>
#include "phtml.h"

#ifdef _UNICODE
typedef std::wstring String;
#else
typedef std::string String;
#endif

static void trim( String *pstr ) {
   while ( _istspace( (*pstr)[ 0 ] ) ) {
      pstr->erase( 0, 1 );
   }
}

static bool getToken(
   String *pstr, bool *pbPara, bool *pbBold, String *pstrWord )
{
   *pstrWord = _T( "" );

   if ( pstr->empty() ) {
      return false;                   //*** FUNCTION EXIT POINT
   }

   const int iSpace   = pstr->find_first_of( _T( " \t\r\n" ) );
   const int iPara    = pstr->find( _T( "<p>" ) );
   const int iBold    = pstr->find( _T( "<b>" ) );
   const int iEndBold = pstr->find( _T( "</b>" ) );

   *pbPara = ( 0 == iPara );
   if ( 0 == iSpace ) {
      *pstrWord = _T( " " );
      trim( pstr );
      *pbPara = 0 == pstr->find( _T( "<p>" ) );
      if ( !*pbPara ) {
         return true;                 //*** FUNCTION EXIT POINT
      }
   }

   if ( *pbPara ) {
      pstr->erase( 0, 3 );
      trim( pstr );
   } else if ( 0 == iBold ) {
      *pbBold = true;
      pstr->erase( 0, 3 );
   } else if ( 0 == iEndBold ) {
      *pbBold = false;
      pstr->erase( 0, 4 );
   } else {                           // It's a word...
      int iNext = pstr->length();     // ...find next token
      if ( 0 < iSpace   && iSpace   < iNext ) { iNext = iSpace  ; }
      if ( 0 < iPara    && iPara    < iNext ) { iNext = iPara   ; }
      if ( 0 < iBold    && iBold    < iNext ) { iNext = iBold   ; }
      if ( 0 < iEndBold && iEndBold < iNext ) { iNext = iEndBold; }

      *pstrWord = pstr->substr( 0, iNext );
      pstr->erase( 0, iNext );
   }
   return true;                       //*** FUNCTION EXIT POINT
}

void paintHTML( HDC hdc, 
   LPCTSTR pszString, RECT *pRect, HFONT hfontNormal )
{
   HFONT hfontBold = hfontNormal;
   LOGFONT logFont = { 0 };
   if ( GetObject( hfontNormal, sizeof logFont, &logFont ) ) {
      logFont.lfWeight = FW_BOLD;
      hfontBold = CreateFontIndirect( &logFont );
   }

   const int nSavedDC = SaveDC( hdc );
   SetTextAlign( hdc, GetTextAlign( hdc ) | TA_UPDATECP );
   HRGN hrgnClip = CreateRectRgnIndirect( pRect );
   SelectClipRgn( hdc, hrgnClip );
   SelectFont( hdc, hfontNormal );

   MoveToEx( hdc, pRect->left, pRect->top, 0 );

   SIZE sizeSpace;
   GetTextExtentPoint32( hdc, _T( "y" ), 2, &sizeSpace );
   const int nLineHeight = sizeSpace.cy;
   const int w = pRect->right - pRect->left;
   const int h = pRect->bottom - pRect->top;

   bool bPara = false;
   bool bBold = false;
   String str( pszString );
   while ( !str.empty() ) {
      POINT ptCurrPos;
      GetCurrentPositionEx( hdc, &ptCurrPos );

      String strWord;
      if ( !getToken( &str, &bPara, &bBold, &strWord ) ) {
         break;                       //*** LOOP EXIT POINT
      }

      if ( bPara ) {
         MoveToEx( hdc,
            pRect->left, ptCurrPos.y + 3 * nLineHeight / 2, 0 );
      } else if ( !strWord.empty() ) {
         SelectFont( hdc, bBold ? hfontBold : hfontNormal );

         SIZE sizeText;
         GetTextExtentPoint32( hdc,
            strWord.c_str(), strWord.length(), &sizeText );
         if ( pRect->left < ptCurrPos.x &&
              pRect->left + w < ptCurrPos.x + sizeText.cx ) 
         {
            MoveToEx( 
               hdc, pRect->left, ptCurrPos.y + nLineHeight, 0 );
            GetCurrentPositionEx( hdc, &ptCurrPos );
         }
         if ( pRect->top + h < ptCurrPos.y + sizeText.cy ) {
            break;                    //*** LOOP EXIT POINT
         }
         if ( pRect->left < ptCurrPos.x || 
              !_istspace( strWord[ 0 ] ) ) 
         {
            TextOut( hdc, 0, 0, strWord.c_str(), strWord.length() );
         }
      }
   }

   RestoreDC( hdc, nSavedDC );
   DeleteRgn( hrgnClip );
   if ( hfontNormal != hfontBold ) {
      DeleteFont( hfontBold );
   }
}

static void onPaint( HWND hwnd ) {
   TCHAR szText[ 1000 ];
   GetWindowText( hwnd, szText, sizeof( TCHAR ) * sizeof szText );

   RECT rc;
   GetClientRect( hwnd, &rc );

   PAINTSTRUCT ps;
   BeginPaint( hwnd, &ps );
   const int nSavedDC = SaveDC( ps.hdc );
   SetTextColor( ps.hdc, GetSysColor( COLOR_BTNTEXT ) );
   SetBkColor  ( ps.hdc, GetSysColor( COLOR_BTNFACE ) );

   paintHTML( ps.hdc, szText, &rc, GetWindowFont( hwnd ) );

   RestoreDC( ps.hdc, nSavedDC );
   EndPaint( hwnd, &ps );
}

static WNDPROC s_savedWndProc;

static LRESULT CALLBACK PocketHTML(
   HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
   LRESULT lResult = 0;
   if ( WM_PAINT == msg ) {
      onPaint( hwnd );
   } else if ( WM_ERASEBKGND == msg ) {
      HDC hdc = reinterpret_cast< HDC >( wParam );
      RECT rc;
      GetClientRect( hwnd, &rc );
      const COLORREF crSaveBkColor =
         SetBkColor( hdc, GetSysColor( COLOR_BTNFACE ) );
      ExtTextOut( hdc, 0, 0, ETO_OPAQUE, &rc, 0, 0, 0 );
      SetBkColor( hdc, crSaveBkColor );
   } else if ( WM_SETTEXT == msg ) {
      SetWindowRedraw( hwnd, FALSE );
      lResult = CallWindowProc(
         s_savedWndProc, hwnd, msg, wParam, lParam );
      SetWindowRedraw( hwnd, TRUE );
      InvalidateRect( hwnd, 0, TRUE );
   } else {
      lResult = CallWindowProc(
         s_savedWndProc, hwnd, msg, wParam, lParam );
   }
   return lResult;
}

void subclassHTML( HWND hwnd ) {
   s_savedWndProc = SubclassWindow( hwnd, PocketHTML );
}
