//======================================================================
// 
// DbgTrapCommon.h
//
// 1999 Jose Flores
//
// 
// Contains definitions, typedefs, etc common both to the GUI & driver
// 
//
//
//======================================================================

#ifndef _DBGTRAP_COMMON_h_
#define _DBGTRAP_COMMON_h_


//======================================================================
//                          M A C R O S 
//======================================================================

#define DT_VERSION  (0x1000)// Current version we're on (High byte is the 
                            // major version. Low byte is the minor vers.)

#define IOCTL_HOOK          CTL_CODE(FILE_DEVICE_UNKNOWN, 0x800, METHOD_NEITHER, FILE_ANY_ACCESS)
#define IOCTL_UNHOOK        CTL_CODE(FILE_DEVICE_UNKNOWN, 0x801, METHOD_NEITHER, FILE_ANY_ACCESS)
#define IOCTL_RESET         CTL_CODE(FILE_DEVICE_UNKNOWN, 0x802, METHOD_NEITHER, FILE_ANY_ACCESS)
#define IOCTL_LOGUSER       CTL_CODE(FILE_DEVICE_UNKNOWN, 0x803, METHOD_NEITHER, FILE_ANY_ACCESS)
#define IOCTL_LOGKERNEL     CTL_CODE(FILE_DEVICE_UNKNOWN, 0x804, METHOD_NEITHER, FILE_ANY_ACCESS)

#define DEFAULT_BUFFER_SIZE   (256*1024)// Max size of the event buffer

#define MAX_PROC_NAME_LENGTH  (24)     // Maximum size of process name string 
                                       // (including NULL terminator)
#define MAX_STRING_LENGTH     (0x100)  // Max length of a DbgPrint string, or symbol name

//
// All  possible debugger services available
//
typedef enum NtDebuggerService
{
    DS_PRINT=0x1,       // DbgPrint 
    DS_PROMPT,          // DebugPromt
    DS_LOADSYMBOLS,     // Load symbols for a system image
    DS_UNLOADSYMBOLS,   // Unload symbols for a system image
    DS_UNKNOWN
};

//======================================================================
//					            T Y P E D E F S
//======================================================================


//
// Our debug service event structure
//
#pragma pack(1)
typedef struct DBGTRAP_EVENT
{
   DWORD          count;   // Number of events that occured before this
   LARGE_INTEGER  time;    // Time this event occured (NT format)
   char           procName[MAX_PROC_NAME_LENGTH];// Name of the process this event occured in
   HANDLE         pid;        // Process ID the event occured in 
   NtDebuggerService action;  // Method (action) that was invoked   

   PVOID          callingAddr;// Return address of method's caller. 

   char           string[MAX_STRING_LENGTH];

} DBGTRAP_EVENT, *PDBGTRAP_EVENT;
#pragma pack()


#pragma pack(1)
#pragma warning( disable: 4200 ) // nonstandard extension used : zero-sized array in struct/union
typedef struct DBGTRAP_HEADER {
    WORD            size;       // size in bytes of this header 
    WORD            version;    // current version of the driver
    DWORD           sig;        // Our signature...should be 'JOSE'
    DWORD           numEntries; // Max # of entries this buffer can hold
    DWORD           index;      // Current index into the buffer

    // The shared event buffer immediately follows this header
    DBGTRAP_EVENT   eb[0];
} DBGTRAP_HEADER, *PDBGTRAP_HEADER;
#pragma warning( default: 4200 )
#pragma pack()

#endif
