// ui.c

#define UNICODE
#define _UNICODE

#include <windows.h>
#include <winwlx.h>
#include "agena.h"

//-----------------------------------------------------------------
BOOL CenterWindow(HWND hWnd)
{
    RECT    rect;
    int     w, h, wScreen, hScreen, xNew, yNew;
    HDC     hDC;

    hDC = GetDC(hWnd);
    GetWindowRect(hWnd, &rect);
    w = rect.right - rect.left;
    h = rect.bottom - rect.top;
    wScreen = GetDeviceCaps(hDC, HORZRES);
    hScreen = GetDeviceCaps(hDC, VERTRES);
    xNew = wScreen/2 - w/2;
    yNew = hScreen/2 - h/2;
    ReleaseDC(hWnd, hDC);

    return SetWindowPos(hWnd, NULL, xNew, yNew, 0, 0, 
                        SWP_NOSIZE | SWP_NOZORDER);
}

//-----------------------------------------------------------------
int CALLBACK WelcomeDlgProc(HWND hDlg, UINT Msg, WPARAM wParam,
                            LPARAM lParam)
{
    switch (Msg) {
        case WM_INITDIALOG:
            CenterWindow(hDlg);
            return TRUE;

        case WLX_WM_SAS:
           DBG1(TEXT("Agena!WelcomeDlgProc,WLX_WM_SAS,%x\n"),wParam);
           break;

        default:
            return FALSE;
    }
    return FALSE;
}

//-----------------------------------------------------------------
int CALLBACK SessionDlgProc(HWND hDlg, UINT Msg, WPARAM wParam,
                            LPARAM lParam)
{
    switch (Msg) {
        case WM_INITDIALOG:
            SetWindowLong(hDlg, GWL_USERDATA, lParam);
            CenterWindow(hDlg);
            SetFocus(GetDlgItem(hDlg, ID_LOGOFF));
            return TRUE;

        case WLX_WM_SAS:
           DBG1(TEXT("Agena!SessionDlgProc,WLX_WM_SAS,%x\n"),wParam);
           break;
        
        case WM_COMMAND:
            switch (LOWORD(wParam)) {
                case ID_LOGOFF:
                    EndDialog(hDlg, WLX_SAS_ACTION_LOGOFF);
                    return TRUE;

                case ID_SHUTDOWN:
                    EndDialog(hDlg, WLX_SAS_ACTION_SHUTDOWN);
                    return TRUE;

                case ID_RESTART:
                    EndDialog(hDlg, WLX_SAS_ACTION_SHUTDOWN_REBOOT);
                    return TRUE;

                case ID_LOCK:
                    EndDialog(hDlg, WLX_SAS_ACTION_LOCK_WKSTA);
                    return TRUE;

                case IDCANCEL:
                    EndDialog(hDlg, WLX_SAS_ACTION_NONE);
                    return TRUE;

                default:
                    break;
            }

        default:
            return FALSE;
    }
    return FALSE;
}

//-----------------------------------------------------------------
int CALLBACK LogonDlgProc(HWND hDlg, UINT Msg, WPARAM wParam,
                          LPARAM lParam)
{
    PAGENA_CONTEXT p;

    switch (Msg) {
        case WM_INITDIALOG:
            SetWindowLong(hDlg, GWL_USERDATA, lParam);
            CenterWindow(hDlg);
            SetFocus(GetDlgItem(hDlg, IDOK));
            return TRUE;

        case WLX_WM_SAS:
            DBG1(TEXT("Agena!LogonDlgProc,WLX_WM_SAS,%x\n"),wParam);
            break;

        case WM_COMMAND:
            p = (PAGENA_CONTEXT)GetWindowLong(hDlg, GWL_USERDATA);

            switch (LOWORD(wParam)) {
                case IDOK:
                    p->UserName[0] = 0;
                    p->Password[0] = 0;
                    p->Domain[0] = 0;
                    GetDlgItemText(hDlg, ID_USERNAME, p->UserName,
                                   MAX_PATH);
                    GetDlgItemText(hDlg, ID_PASSWORD, p->Password, 
                                   MAX_PATH);
                    GetDlgItemText(hDlg, ID_DOMAIN, p->Domain, 
                                   MAX_PATH);

                    if (AgenaLogonUser(p)) {
                        EndDialog(hDlg, WLX_SAS_ACTION_LOGON);
                    } else {
                        MessageBeep(0);
                    }
                    return TRUE;

                case ID_SHUTDOWN:
                    EndDialog(hDlg, WLX_SAS_ACTION_SHUTDOWN);
                    return TRUE;
                
                case ID_RESTART:
                    EndDialog(hDlg, WLX_SAS_ACTION_SHUTDOWN_REBOOT);
                    return TRUE;

                case IDCANCEL:
                    EndDialog(hDlg, WLX_SAS_ACTION_NONE);
                    return TRUE;

                default:
                    break;
            }

        default:
            return FALSE;
    }
    return FALSE;
}

//-----------------------------------------------------------------
int CALLBACK UnlockDlgProc(HWND hDlg, UINT Msg, WPARAM wParam,
                           LPARAM lParam)
{
    PAGENA_CONTEXT p;
    WCHAR szUserName[256], szPassword[256], szDomain[256];

    switch (Msg) {
        case WM_INITDIALOG:
            SetWindowLong(hDlg, GWL_USERDATA, lParam);
            CenterWindow(hDlg);
            SetFocus(GetDlgItem(hDlg, IDOK));
            return TRUE;

        case WLX_WM_SAS:
            DBG1(TEXT("Agena!UnlockDlgProc,WLX_WM_SAS,%x\n"),wParam);
            break;

        case WM_COMMAND:
            p = (PAGENA_CONTEXT)GetWindowLong(hDlg, GWL_USERDATA);

            switch (LOWORD(wParam)) {
                case IDOK:
                    szUserName[0] = szPassword[0] = szDomain[0] = 0;
                    GetDlgItemText(hDlg,ID_USERNAME,szUserName,256);
                    GetDlgItemText(hDlg,ID_PASSWORD,szPassword,256);
                    GetDlgItemText(hDlg,ID_DOMAIN,szDomain,256);
                    if ((wcscmp(p->UserName, szUserName) == 0) &&
                        (wcscmp(p->Password, szPassword) == 0) &&
                        (wcscmp(p->Domain, szDomain) == 0)) {
                        EndDialog(hDlg, WLX_SAS_ACTION_UNLOCK_WKSTA);
                        return TRUE;
                    } else {
                        MessageBeep(0);
                    }
                    break;

                case IDCANCEL:
                    EndDialog(hDlg, WLX_SAS_ACTION_NONE);
                    return TRUE;

                default:
                    break;
            }

        default:
            return FALSE;
    }
    return FALSE;
}

